@@ -34,6 +34,20 @@ if sys.version_info[0] >= 3:
     unicode = str
 
 
+def load_module(name):
+    if os.path.exists(name) and os.path.splitext(name)[1] == '.py':
+        sys.path.insert(0, os.path.dirname(os.path.abspath(name)))
+        try:
+            m = os.path.splitext(os.path.basename(name))[0]
+            module = importlib.import_module(m)
+        finally:
+            sys.path.pop(0)
+
+        return module
+
+    return importlib.import_module('dbusmock.templates.' + name)
+
+
 class DBusMockObject(dbus.service.Object):
     '''Mock D-Bus object
 
@@ -82,6 +96,8 @@ class DBusMockObject(dbus.service.Object):
     def Get(self, interface_name, property_name):
         '''Standard D-Bus API for getting a property value'''
 
+        if not interface_name:
+            interface_name = self.interface
         try:
             return self.GetAll(interface_name)[property_name]
         except KeyError:
@@ -94,6 +110,8 @@ class DBusMockObject(dbus.service.Object):
     def GetAll(self, interface_name, *args, **kwargs):
         '''Standard D-Bus API for getting all property values'''
 
+        if not interface_name:
+            interface_name = self.interface
         try:
             return self.props[interface_name]
         except KeyError:
@@ -292,8 +310,9 @@ class DBusMockObject(dbus.service.Object):
         properties for the tests, and not the skeleton of common properties,
         interfaces, and methods.
 
-        template: Name of the template to load. See "pydoc dbusmock.templates"
-                  for a list of available templates, and
+        template: Name of the template to load or the full path to a *.py file
+                  for custom templates. See "pydoc dbusmock.templates" for a
+                  list of available templates from python-dbusmock package, and
                   "pydoc dbusmock.templates.NAME" for documentation about
                   template NAME.
         parameters: A parameter (string) → value (variant) map, for
@@ -302,7 +321,7 @@ class DBusMockObject(dbus.service.Object):
                     details.
         '''
         try:
-            module = importlib.import_module('dbusmock.templates.' + template)
+            module = load_module(template)
         except ImportError as e:
             raise dbus.exceptions.DBusException('Cannot add template %s: %s' % (template, str(e)))
 
@@ -381,6 +400,10 @@ class DBusMockObject(dbus.service.Object):
 
         self.call_log = []
 
+    @dbus.service.signal(MOCK_IFACE, signature='sav')
+    def MethodCalled(self, name, args):
+        pass
+
     def mock_method(self, interface, dbus_method, in_signature, *args, **kwargs):
         '''Master mock method.
 
@@ -400,6 +423,7 @@ class DBusMockObject(dbus.service.Object):
 
         self.log(dbus_method + self.format_args(args))
         self.call_log.append((int(time.time()), str(dbus_method), args))
+        self.MethodCalled(dbus_method, args)
 
         code = self.methods[interface][dbus_method][2]
         if code:
