@@ -14,6 +14,8 @@ __license__ = 'LGPL 3+'
 import unittest
 import sys
 import tempfile
+import subprocess
+import time
 
 import dbus
 import dbus.mainloop.glib
@@ -476,6 +478,89 @@ assert args[2] == 5
         self.assertGreater(mock_calls[1][0], 10000)  # timestamp
         self.assertEqual(mock_calls[1][1], ['bar'])
 
+    def test_dbus_method_called(self):
+        '''subscribe to MethodCalled signal'''
+
+        loop = GLib.MainLoop()
+        caught_signals = []
+
+        def method_called(method, args, **kwargs):
+            caught_signals.append((method, args))
+            loop.quit()
+
+        self.dbus_mock.AddMethod('', 'Do', 's', '', '')
+        self.dbus_mock.connect_to_signal('MethodCalled', method_called)
+        self.assertEqual(self.dbus_test.Do('foo'), None)
+
+        GLib.timeout_add(5000, loop.quit)
+        loop.run()
+
+        self.assertEqual(len(caught_signals), 1)
+        method, args = caught_signals[0]
+        self.assertEqual(method, 'Do')
+        self.assertEqual(len(args), 1)
+        self.assertEqual(args[0], 'foo')
+
+
+class TestTemplates(dbusmock.DBusTestCase):
+    '''Test template API'''
+
+    @classmethod
+    def setUpClass(klass):
+        klass.start_session_bus()
+
+    def test_local(self):
+        '''Load a local template *.py file'''
+
+        with tempfile.NamedTemporaryFile(prefix='answer_', suffix='.py') as my_template:
+            my_template.write(b'''import dbus
+BUS_NAME = 'universe.Ultimate'
+MAIN_OBJ = '/'
+MAIN_IFACE = 'universe.Ultimate'
+SYSTEM_BUS = False
+
+def load(mock, parameters):
+    mock.AddMethods(MAIN_IFACE, [('Answer', '', 'i', 'ret = 42')])
+''')
+            my_template.flush()
+            (p_mock, dbus_ultimate) = self.spawn_server_template(
+                my_template.name, stdout=subprocess.PIPE)
+
+        self.assertEqual(dbus_ultimate.Answer(), 42)
+
+    def test_local_nonexisting(self):
+        self.assertRaises(ImportError, self.spawn_server_template, '/non/existing.py')
+
+
+class TestCleanup(dbusmock.DBusTestCase):
+    '''Test cleanup of resources'''
+
+    def test_mock_terminates_with_bus(self):
+        '''Spawned mock processes exit when bus goes down'''
+
+        self.start_session_bus()
+        p_mock = self.spawn_server('org.freedesktop.Test',
+                                   '/',
+                                   'org.freedesktop.Test.Main')
+        self.stop_dbus(self.session_bus_pid)
+
+        # give the mock 2 seconds to terminate
+        timeout = 20
+        while timeout > 0:
+            if p_mock.poll() is not None:
+                break
+            timeout -= 1
+            time.sleep(0.1)
+
+        if p_mock.poll() is None:
+            # clean up manually
+            p_mock.terminate()
+            p_mock.wait()
+            self.fail('mock process did not terminate after 2 seconds')
+
+        self.assertEqual(p_mock.wait(), 0)
+
+
 if __name__ == '__main__':
     # avoid writing to stderr
     unittest.main(testRunner=unittest.TextTestRunner(stream=sys.stdout, verbosity=2))
